package com.sun.org.apache.xerces.internal.jaxp;

import org.xml.sax.SAXParseException;
import org.xml.sax.ErrorHandler;
import com.sun.org.apache.xerces.internal.util.LocatorProxy;
import com.sun.org.apache.xerces.internal.xni.XMLLocator;
import com.sun.org.apache.xerces.internal.util.AttributesProxy;
import com.sun.org.apache.xerces.internal.xni.NamespaceContext;
import org.xml.sax.ContentHandler;
import com.sun.org.apache.xerces.internal.impl.xs.opti.DefaultXMLDocumentHandler;
import com.sun.org.apache.xerces.internal.xni.XMLDocumentHandler;
import org.xml.sax.SAXException;
import com.sun.org.apache.xerces.internal.util.AugmentationsImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.w3c.dom.TypeInfo;
import org.xml.sax.Attributes;
import com.sun.org.apache.xerces.internal.xni.parser.XMLConfigurationException;
import com.sun.org.apache.xerces.internal.xni.parser.XMLComponentManager;
import com.sun.org.apache.xerces.internal.xni.XMLString;
import com.sun.org.apache.xerces.internal.xni.QName;
import com.sun.org.apache.xerces.internal.xni.parser.XMLInputSource;
import java.io.IOException;
import com.sun.org.apache.xerces.internal.xni.XNIException;
import com.sun.org.apache.xerces.internal.dom.DOMInputImpl;
import com.sun.org.apache.xerces.internal.util.XMLResourceIdentifierImpl;
import org.w3c.dom.ls.LSInput;
import org.w3c.dom.ls.LSResourceResolver;
import com.sun.org.apache.xerces.internal.util.ErrorHandlerWrapper;
import com.sun.org.apache.xerces.internal.xni.parser.XMLErrorHandler;
import com.sun.org.apache.xerces.internal.util.ErrorHandlerProxy;
import com.sun.org.apache.xerces.internal.xni.parser.XMLEntityResolver;
import com.sun.org.apache.xerces.internal.impl.XMLErrorReporter;
import com.sun.org.apache.xerces.internal.util.SymbolTable;
import com.sun.org.apache.xerces.internal.xni.XMLAttributes;
import com.sun.org.apache.xerces.internal.xni.Augmentations;
import javax.xml.validation.TypeInfoProvider;
import javax.xml.validation.ValidatorHandler;
import com.sun.org.apache.xerces.internal.xni.parser.XMLComponent;

final class JAXPValidatorComponent extends TeeXMLDocumentFilterImpl implements XMLComponent
{
    private static final String ENTITY_MANAGER = "http://apache.org/xml/properties/internal/entity-manager";
    private static final String ERROR_REPORTER = "http://apache.org/xml/properties/internal/error-reporter";
    private static final String SYMBOL_TABLE = "http://apache.org/xml/properties/internal/symbol-table";
    private final ValidatorHandler validator;
    private final JAXPValidatorComponent.XNI2SAX xni2sax;
    private final JAXPValidatorComponent.SAX2XNI sax2xni;
    private final TypeInfoProvider typeInfoProvider;
    private Augmentations fCurrentAug;
    private XMLAttributes fCurrentAttributes;
    private SymbolTable fSymbolTable;
    private XMLErrorReporter fErrorReporter;
    private XMLEntityResolver fEntityResolver;
    private static final TypeInfoProvider noInfoProvider;
    
    public JAXPValidatorComponent(ValidatorHandler validatorHandler) {
        this.xni2sax = new JAXPValidatorComponent.XNI2SAX();
        this.sax2xni = new JAXPValidatorComponent.SAX2XNI();
        this.validator = validatorHandler;
        TypeInfoProvider tip = validatorHandler.getTypeInfoProvider();
        if (tip == null) {
            tip = JAXPValidatorComponent.noInfoProvider;
        }
        this.typeInfoProvider = tip;
        this.xni2sax.setContentHandler(this.validator);
        this.validator.setContentHandler(this.sax2xni);
        this.setSide(this.xni2sax);
        this.validator.setErrorHandler(new ErrorHandlerProxy() {
            @Override
            protected XMLErrorHandler getErrorHandler() {
                XMLErrorHandler handler = JAXPValidatorComponent.this.fErrorReporter.getErrorHandler();
                if (handler != null) {
                    return handler;
                }
                return new ErrorHandlerWrapper(JAXPValidatorComponent.DraconianErrorHandler.getInstance());
            }
        });
        this.validator.setResourceResolver(new LSResourceResolver() {
            @Override
            public LSInput resolveResource(String type, String ns, String publicId, String systemId, String baseUri) {
                if (JAXPValidatorComponent.this.fEntityResolver == null) {
                    return null;
                }
                try {
                    XMLInputSource is = JAXPValidatorComponent.this.fEntityResolver.resolveEntity(new XMLResourceIdentifierImpl(publicId, systemId, baseUri, null));
                    if (is == null) {
                        return null;
                    }
                    LSInput di = new DOMInputImpl();
                    di.setBaseURI(is.getBaseSystemId());
                    di.setByteStream(is.getByteStream());
                    di.setCharacterStream(is.getCharacterStream());
                    di.setEncoding(is.getEncoding());
                    di.setPublicId(is.getPublicId());
                    di.setSystemId(is.getSystemId());
                    return di;
                }
                catch (IOException e) {
                    throw new XNIException(e);
                }
            }
        });
    }
    
    @Override
    public void startElement(QName element, XMLAttributes attributes, Augmentations augs) throws XNIException {
        this.fCurrentAttributes = attributes;
        this.fCurrentAug = augs;
        this.xni2sax.startElement(element, attributes, null);
        this.fCurrentAttributes = null;
    }
    
    @Override
    public void endElement(QName element, Augmentations augs) throws XNIException {
        this.fCurrentAug = augs;
        this.xni2sax.endElement(element, null);
    }
    
    @Override
    public void emptyElement(QName element, XMLAttributes attributes, Augmentations augs) throws XNIException {
        this.startElement(element, attributes, augs);
        this.endElement(element, augs);
    }
    
    @Override
    public void characters(XMLString text, Augmentations augs) throws XNIException {
        this.fCurrentAug = augs;
        this.xni2sax.characters(text, null);
    }
    
    @Override
    public void ignorableWhitespace(XMLString text, Augmentations augs) throws XNIException {
        this.fCurrentAug = augs;
        this.xni2sax.ignorableWhitespace(text, null);
    }
    
    @Override
    public void reset(XMLComponentManager componentManager) throws XMLConfigurationException {
        this.fSymbolTable = (SymbolTable)componentManager.getProperty("http://apache.org/xml/properties/internal/symbol-table");
        this.fErrorReporter = (XMLErrorReporter)componentManager.getProperty("http://apache.org/xml/properties/internal/error-reporter");
        try {
            this.fEntityResolver = (XMLEntityResolver)componentManager.getProperty("http://apache.org/xml/properties/internal/entity-manager");
        }
        catch (XMLConfigurationException e) {
            this.fEntityResolver = null;
        }
    }
    
    private void updateAttributes(Attributes atts) {
        for (int len = atts.getLength(), i = 0; i < len; ++i) {
            String aqn = atts.getQName(i);
            int j = this.fCurrentAttributes.getIndex(aqn);
            String av = atts.getValue(i);
            if (j == -1) {
                int idx = aqn.indexOf(58);
                String prefix;
                if (idx < 0) {
                    prefix = null;
                }
                else {
                    prefix = this.symbolize(aqn.substring(0, idx));
                }
                j = this.fCurrentAttributes.addAttribute(new QName(prefix, this.symbolize(atts.getLocalName(i)), this.symbolize(aqn), this.symbolize(atts.getURI(i))), atts.getType(i), av);
            }
            else if (!av.equals(this.fCurrentAttributes.getValue(j))) {
                this.fCurrentAttributes.setValue(j, av);
            }
        }
    }
    
    private String symbolize(String s) {
        return this.fSymbolTable.addSymbol(s);
    }
    
    @Override
    public String[] getRecognizedFeatures() {
        return null;
    }
    
    @Override
    public void setFeature(String featureId, boolean state) throws XMLConfigurationException {
    }
    
    @Override
    public String[] getRecognizedProperties() {
        return new String[] { "http://apache.org/xml/properties/internal/entity-manager", "http://apache.org/xml/properties/internal/error-reporter", "http://apache.org/xml/properties/internal/symbol-table" };
    }
    
    @Override
    public void setProperty(String propertyId, Object value) throws XMLConfigurationException {
    }
    
    @Override
    public Boolean getFeatureDefault(String featureId) {
        return null;
    }
    
    @Override
    public Object getPropertyDefault(String propertyId) {
        return null;
    }
    
    static void access$400(JAXPValidatorComponent x0, Attributes x1) {
        x0.updateAttributes(x1);
    }
    
    static String access$700(JAXPValidatorComponent x0, String x1) {
        return x0.symbolize(x1);
    }
    
    static {
        noInfoProvider = new TypeInfoProvider() {
            @Override
            public TypeInfo getElementTypeInfo() {
                return null;
            }
            
            @Override
            public TypeInfo getAttributeTypeInfo(int index) {
                return null;
            }
            
            public TypeInfo getAttributeTypeInfo(String attributeQName) {
                return null;
            }
            
            public TypeInfo getAttributeTypeInfo(String attributeUri, String attributeLocalName) {
                return null;
            }
            
            @Override
            public boolean isIdAttribute(int index) {
                return false;
            }
            
            @Override
            public boolean isSpecified(int index) {
                return false;
            }
        };
    }
    
    private final class SAX2XNI extends DefaultHandler
    {
        private final Augmentations fAugmentations;
        private final QName fQName;
        
        private SAX2XNI() {
            this.fAugmentations = new AugmentationsImpl();
            this.fQName = new QName();
        }
        
        @Override
        public void characters(char[] ch, int start, int len) throws SAXException {
            try {
                this.handler().characters(new XMLString(ch, start, len), this.aug());
            }
            catch (XNIException e) {
                throw this.toSAXException(e);
            }
        }
        
        @Override
        public void ignorableWhitespace(char[] ch, int start, int len) throws SAXException {
            try {
                this.handler().ignorableWhitespace(new XMLString(ch, start, len), this.aug());
            }
            catch (XNIException e) {
                throw this.toSAXException(e);
            }
        }
        
        @Override
        public void startElement(String uri, String localName, String qname, Attributes atts) throws SAXException {
            try {
                JAXPValidatorComponent.access$400(JAXPValidatorComponent.this, atts);
                this.handler().startElement(this.toQName(uri, localName, qname), JAXPValidatorComponent.this.fCurrentAttributes, this.elementAug());
            }
            catch (XNIException e) {
                throw this.toSAXException(e);
            }
        }
        
        @Override
        public void endElement(String uri, String localName, String qname) throws SAXException {
            try {
                this.handler().endElement(this.toQName(uri, localName, qname), this.aug());
            }
            catch (XNIException e) {
                throw this.toSAXException(e);
            }
        }
        
        private Augmentations elementAug() {
            Augmentations aug = this.aug();
            return aug;
        }
        
        private Augmentations aug() {
            if (JAXPValidatorComponent.this.fCurrentAug != null) {
                Augmentations r = JAXPValidatorComponent.this.fCurrentAug;
                JAXPValidatorComponent.this.fCurrentAug = null;
                return r;
            }
            this.fAugmentations.removeAllItems();
            return this.fAugmentations;
        }
        
        private XMLDocumentHandler handler() {
            return JAXPValidatorComponent.this.getDocumentHandler();
        }
        
        private SAXException toSAXException(XNIException xe) {
            Exception e = xe.getException();
            if (e == null) {
                e = xe;
            }
            if ((e instanceof SAXException)) {
                return (SAXException)e;
            }
            return new SAXException(e);
        }
        
        private QName toQName(String uri, String localName, String qname) {
            String prefix = null;
            int idx = qname.indexOf(58);
            if (idx > 0) {
                prefix = JAXPValidatorComponent.access$700(JAXPValidatorComponent.this, qname.substring(0, idx));
            }
            localName = JAXPValidatorComponent.access$700(JAXPValidatorComponent.this, localName);
            qname = JAXPValidatorComponent.access$700(JAXPValidatorComponent.this, qname);
            uri = JAXPValidatorComponent.access$700(JAXPValidatorComponent.this, uri);
            this.fQName.setValues(prefix, localName, qname, uri);
            return this.fQName;
        }
    }
    
    private final class XNI2SAX extends DefaultXMLDocumentHandler
    {
        private ContentHandler fContentHandler;
        private String fVersion;
        protected NamespaceContext fNamespaceContext;
        private final AttributesProxy fAttributesProxy;
        
        private XNI2SAX() {
            this.fAttributesProxy = new AttributesProxy(null);
        }
        
        public void setContentHandler(ContentHandler handler) {
            this.fContentHandler = handler;
        }
        
        public ContentHandler getContentHandler() {
            return this.fContentHandler;
        }
        
        @Override
        public void xmlDecl(String version, String encoding, String standalone, Augmentations augs) throws XNIException {
            this.fVersion = version;
        }
        
        @Override
        public void startDocument(XMLLocator locator, String encoding, NamespaceContext namespaceContext, Augmentations augs) throws XNIException {
            this.fNamespaceContext = namespaceContext;
            this.fContentHandler.setDocumentLocator(new LocatorProxy(locator));
            try {
                this.fContentHandler.startDocument();
            }
            catch (SAXException e) {
                throw new XNIException(e);
            }
        }
        
        @Override
        public void endDocument(Augmentations augs) throws XNIException {
            try {
                this.fContentHandler.endDocument();
            }
            catch (SAXException e) {
                throw new XNIException(e);
            }
        }
        
        @Override
        public void processingInstruction(String target, XMLString data, Augmentations augs) throws XNIException {
            try {
                this.fContentHandler.processingInstruction(target, data.toString());
            }
            catch (SAXException e) {
                throw new XNIException(e);
            }
        }
        
        @Override
        public void startElement(QName element, XMLAttributes attributes, Augmentations augs) throws XNIException {
            try {
                int count = this.fNamespaceContext.getDeclaredPrefixCount();
                if (count > 0) {
                    String prefix = null;
                    String uri = null;
                    for (int i = 0; i < count; ++i) {
                        prefix = this.fNamespaceContext.getDeclaredPrefixAt(i);
                        uri = this.fNamespaceContext.getURI(prefix);
                        this.fContentHandler.startPrefixMapping(prefix, (uri == null) ? "" : uri);
                    }
                }
                String uri2 = (element.uri != null) ? element.uri : "";
                String localpart = element.localpart;
                this.fAttributesProxy.setAttributes(attributes);
                this.fContentHandler.startElement(uri2, localpart, element.rawname, this.fAttributesProxy);
            }
            catch (SAXException e) {
                throw new XNIException(e);
            }
        }
        
        @Override
        public void endElement(QName element, Augmentations augs) throws XNIException {
            try {
                String uri = (element.uri != null) ? element.uri : "";
                String localpart = element.localpart;
                this.fContentHandler.endElement(uri, localpart, element.rawname);
                int count = this.fNamespaceContext.getDeclaredPrefixCount();
                if (count > 0) {
                    for (int i = 0; i < count; ++i) {
                        this.fContentHandler.endPrefixMapping(this.fNamespaceContext.getDeclaredPrefixAt(i));
                    }
                }
            }
            catch (SAXException e) {
                throw new XNIException(e);
            }
        }
        
        @Override
        public void emptyElement(QName element, XMLAttributes attributes, Augmentations augs) throws XNIException {
            this.startElement(element, attributes, augs);
            this.endElement(element, augs);
        }
        
        @Override
        public void characters(XMLString text, Augmentations augs) throws XNIException {
            try {
                this.fContentHandler.characters(text.ch, text.offset, text.length);
            }
            catch (SAXException e) {
                throw new XNIException(e);
            }
        }
        
        @Override
        public void ignorableWhitespace(XMLString text, Augmentations augs) throws XNIException {
            try {
                this.fContentHandler.ignorableWhitespace(text.ch, text.offset, text.length);
            }
            catch (SAXException e) {
                throw new XNIException(e);
            }
        }
    }
    
    private static final class DraconianErrorHandler implements ErrorHandler
    {
        private static final JAXPValidatorComponent.DraconianErrorHandler ERROR_HANDLER_INSTANCE;
        
        public static JAXPValidatorComponent.DraconianErrorHandler getInstance() {
            return JAXPValidatorComponent.DraconianErrorHandler.ERROR_HANDLER_INSTANCE;
        }
        
        @Override
        public void warning(SAXParseException e) throws SAXException {
        }
        
        @Override
        public void error(SAXParseException e) throws SAXException {
            throw e;
        }
        
        @Override
        public void fatalError(SAXParseException e) throws SAXException {
            throw e;
        }
        
        static {
            ERROR_HANDLER_INSTANCE = new JAXPValidatorComponent.DraconianErrorHandler();
        }
    }
}
